#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QPen>

#if QWT_VERSION < 0x060000
#include "qdata5.h"
#else
#include "qdata.h"
#endif

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    //   
    ui->myPlot->setTitle("QwtPolarPlot Demo");

    //  
    leg = new QwtLegend;
    //   
    leg->setItemMode(QwtLegend::ReadOnlyItem);
    //       
    ui->myPlot->insertLegend(leg,QwtPolarPlot::BottomLegend);

    //  
    grid = new QwtPolarGrid();
    //   
    for (int sc=0; sc < QwtPolar::ScaleCount; sc++)
    {
        //    ,    
        grid->showMinorGrid(sc);
        //  ,       
        grid->setMajorGridPen(sc,QPen(Qt::lightGray));                  //  
        grid->setMinorGridPen(sc,QPen(Qt::lightGray,0,Qt::DotLine));    //  
    }
    //     
    grid->showAxis(QwtPolar::AxisRight,false);
    //     
    grid->showAxis(QwtPolar::AxisTop,true);

    //     
    grid->attach(ui->myPlot);

    //     
    ui->myPlot->setScale(QwtPolar::Radius,0,1,0.2);     //  
    ui->myPlot->setScale(QwtPolar::Azimuth,0,360,30);   //  

    //      "DNA 1"
    curv1 = new QwtPolarCurve(QString("DNA 1"));
    //   2    - -
    curv1->setPen(QPen(Qt::darkGreen,2));

    //      "DNA 2"
    curv2 = new QwtPolarCurve(QString("DNA 2"));
    //    - 
    curv2->setPen(QPen(Qt::red));

    //  
    const int N1 = 361;     //   
    const int N2 = 1441;    //   

    //       (   )
    double *X1 = (double *)malloc((2*N1+2*N2)*sizeof(double));
    double *Y1 = X1 + N1;
    double *X2 = Y1 + N1;
    double *Y2 = X2 + N2;

    //     
    // X1    0  360
    double d = 40;          //  
    double kd = 2.7831 / d;
    //    
    double h = 360./(N1-1);
    for (int k = 0; k < N1; k++)
    {
        double fi = k*h;
        X1[k] = fi;
        double x = fi;
        //     -180 - +180
        while (x >= 180) x -= 360;
        double y = 1;       //  
        if (x != 0) y = qAbs(sin(kd*x)/(kd*x));
        Y1[k] = y;
    }
    //     
    // X2    0  360
    const double fi0 = 30;  //   
    d = 6;                  //  
    kd = 2.7831 / d;
    //    
    h = 360./(N2-1);
    for (int k = 0; k < N2; k++)
    {
        double fi = k*h;
        X2[k] = fi;
        double x = fi - fi0;
        //     -180 - +180
        while (x >= 180) x -= 360;
        double y = 1;       //  
        if (x != 0) y = qAbs(sin(kd*x)/(kd*x));
        Y2[k] = y;
    }

    //    
#if QWT_VERSION < 0x060000
    curv1->setData(QData5(X1,Y1,N1));
    curv2->setData(QData5(X2,Y2,N2));
#else
    curv1->setData(new QData(X1,Y1,N1));
    curv2->setData(new QData(X2,Y2,N2));
#endif

    //   
    free((void *)X1);

    //    
    curv1->attach(ui->myPlot);
    curv2->attach(ui->myPlot);

    //   
    pan = new QwtPolarPanner(ui->myPlot->canvas());
    //   
    zoom = new QwtPolarMagnifier(ui->myPlot->canvas());
}

MainWindow::~MainWindow()
{
    //  ,     :
    delete pan;     //  
    delete zoom;    //  

    delete curv1;   //  
    delete curv2;   //  
    delete grid;    //  
    delete leg;     // 

    delete ui;
}
